@@ -170,8 +170,9 @@ class BaseCli(dnf.Base):
                 switchedModules = dict(self._moduleContainer.getSwitchedStreams())
                 if switchedModules:
                     report_module_switch(switchedModules)
-                    msg = _("It is not possible to switch enabled streams of a module.\n"
-                            "It is recommended to remove all installed content from the module, and "
+                    msg = _("It is not possible to switch enabled streams of a module unless explicitly "
+                            "enabled via configuration option module_stream_switch.\n"
+                            "It is recommended to rather remove all installed content from the module, and "
                             "reset the module using '{prog} module reset <module_name>' command. After "
                             "you reset the module, you can install the other stream.").format(
                         prog=dnf.util.MAIN_PROG)
@@ -252,8 +253,12 @@ class BaseCli(dnf.Base):
             trans = None
 
         if trans:
-            msg = self.output.post_transaction_output(trans)
-            logger.info(msg)
+            # the post transaction summary is already written to log during
+            # Base.do_transaction() so here only print the messages to the
+            # user arranged in columns
+            print()
+            print('\n'.join(self.output.post_transaction_output(trans)))
+            print()
             for tsi in trans:
                 if tsi.state == libdnf.transaction.TransactionItemState_ERROR:
                     raise dnf.exceptions.Error(_('Transaction failed'))
@@ -287,7 +292,7 @@ class BaseCli(dnf.Base):
                 fn = lambda x, y, z: self.output.userconfirm()
                 try:
                     self._get_key_for_package(po, fn)
-                except dnf.exceptions.Error as e:
+                except (dnf.exceptions.Error, ValueError) as e:
                     error_messages.append(str(e))
 
             else:
@@ -502,7 +507,7 @@ class BaseCli(dnf.Base):
             # XXX put this into the ListCommand at some point
             if len(ypl.obsoletes) > 0 and basecmd == 'list':
             # if we've looked up obsolete lists and it's a list request
-                rop = [0, '']
+                rop = len(ypl.obsoletes)
                 print(_('Obsoleting Packages'))
                 for obtup in sorted(ypl.obsoletesTuples,
                                     key=operator.itemgetter(0)):
@@ -514,8 +519,7 @@ class BaseCli(dnf.Base):
             rrap = self.output.listPkgs(ypl.recent, _('Recently Added Packages'),
                                  basecmd, columns=columns)
             if len(patterns) and \
-                rrap[0] and rop[0] and rup[0] and rep[0] and rap[0] and \
-                raep[0] and rip[0]:
+                    rrap == 0 and rop == 0 and rup == 0 and rep == 0 and rap == 0 and raep == 0 and rip == 0:
                 raise dnf.exceptions.Error(_('No matching Packages to list'))
 
     def returnPkgLists(self, pkgnarrow='all', patterns=None,
@@ -597,7 +601,9 @@ class BaseCli(dnf.Base):
         self.conf.showdupesfromrepos = old_sdup
 
         if not matches:
-            raise dnf.exceptions.Error(_('No Matches found'))
+            raise dnf.exceptions.Error(_('No matches found. If searching for a file, '
+                                         'try specifying the full path or using a '
+                                         'wildcard prefix ("*/") at the beginning.'))
 
     def _promptWanted(self):
         # shortcut for the always-off/always-on options
@@ -605,109 +611,6 @@ class BaseCli(dnf.Base):
             return False
         return True
 
-    def _history_get_transactions(self, extcmds):
-        if not extcmds:
-            logger.critical(_('No transaction ID given'))
-            return None
-
-        old = self.history.old(extcmds)
-        if not old:
-            logger.critical(_('Not found given transaction ID'))
-            return None
-        return old
-
-    def history_get_transaction(self, extcmds):
-        old = self._history_get_transactions(extcmds)
-        if old is None:
-            return None
-        if len(old) > 1:
-            logger.critical(_('Found more than one transaction ID!'))
-        return old[0]
-
-    def history_rollback_transaction(self, extcmd):
-        """Rollback given transaction."""
-        old = self.history_get_transaction((extcmd,))
-        if old is None:
-            return 1, ['Failed history rollback, no transaction']
-        last = self.history.last()
-        if last is None:
-            return 1, ['Failed history rollback, no last?']
-        if old.tid == last.tid:
-            return 0, ['Rollback to current, nothing to do']
-
-        mobj = None
-        for trans in self.history.old(list(range(old.tid + 1, last.tid + 1))):
-            if trans.altered_lt_rpmdb:
-                logger.warning(_('Transaction history is incomplete, before %u.'), trans.tid)
-            elif trans.altered_gt_rpmdb:
-                logger.warning(_('Transaction history is incomplete, after %u.'), trans.tid)
-
-            if mobj is None:
-                mobj = dnf.db.history.MergedTransactionWrapper(trans)
-            else:
-                mobj.merge(trans)
-
-        tm = dnf.util.normalize_time(old.beg_timestamp)
-        print("Rollback to transaction %u, from %s" % (old.tid, tm))
-        print(self.output.fmtKeyValFill("  Undoing the following transactions: ",
-                                        ", ".join((str(x) for x in mobj.tids()))))
-        self.output.historyInfoCmdPkgsAltered(mobj)  # :todo
-
-#        history = dnf.history.open_history(self.history)  # :todo
-#        m = libdnf.transaction.MergedTransaction()
-
-#        return
-
-#        operations = dnf.history.NEVRAOperations()
-#        for id_ in range(old.tid + 1, last.tid + 1):
-#            operations += history.transaction_nevra_ops(id_)
-
-        try:
-            self._history_undo_operations(mobj, old.tid + 1, True, strict=self.conf.strict)
-        except dnf.exceptions.PackagesNotInstalledError as err:
-            raise
-            logger.info(_('No package %s installed.'),
-                        self.output.term.bold(ucd(err.pkg_spec)))
-            return 1, ['A transaction cannot be undone']
-        except dnf.exceptions.PackagesNotAvailableError as err:
-            raise
-            logger.info(_('No package %s available.'),
-                        self.output.term.bold(ucd(err.pkg_spec)))
-            return 1, ['A transaction cannot be undone']
-        except dnf.exceptions.MarkingError:
-            raise
-            assert False
-        else:
-            return 2, ["Rollback to transaction %u" % (old.tid,)]
-
-    def history_undo_transaction(self, extcmd):
-        """Undo given transaction."""
-        old = self.history_get_transaction((extcmd,))
-        if old is None:
-            return 1, ['Failed history undo']
-
-        tm = dnf.util.normalize_time(old.beg_timestamp)
-        msg = _("Undoing transaction {}, from {}").format(old.tid, ucd(tm))
-        logger.info(msg)
-        self.output.historyInfoCmdPkgsAltered(old)  # :todo
-
-
-        mobj = dnf.db.history.MergedTransactionWrapper(old)
-
-        try:
-            self._history_undo_operations(mobj, old.tid, strict=self.conf.strict)
-        except dnf.exceptions.PackagesNotInstalledError as err:
-            logger.info(_('No package %s installed.'),
-                        self.output.term.bold(ucd(err.pkg_spec)))
-            return 1, ['An operation cannot be undone']
-        except dnf.exceptions.PackagesNotAvailableError as err:
-            logger.info(_('No package %s available.'),
-                        self.output.term.bold(ucd(err.pkg_spec)))
-            return 1, ['An operation cannot be undone']
-        except dnf.exceptions.MarkingError:
-            raise
-        else:
-            return 2, ["Undoing transaction %u" % (old.tid,)]
 
 class Cli(object):
     def __init__(self, base):
@@ -909,7 +812,7 @@ class Cli(object):
         if opts.destdir is not None:
             self.base.conf.destdir = opts.destdir
             if not self.base.conf.downloadonly and opts.command not in (
-                    'download', 'system-upgrade', 'reposync'):
+                    'download', 'system-upgrade', 'reposync', 'modulesync'):
                 logger.critical(_('--destdir or --downloaddir must be used with --downloadonly '
                                   'or download or system-upgrade command.')
                 )
@@ -1049,43 +952,30 @@ class Cli(object):
         timer()
         return conf
 
-    def _populate_update_security_filter(self, opts, query, cmp_type='eq', all=None):
+    def _populate_update_security_filter(self, opts, cmp_type='eq', all=None):
         """
 
         :param opts:
-        :param query: base package set for filters
-        :param cmp_type: string like "eq", "gt", "gte", "lt", "lte"
+        :param cmp_type: string supported "eq", "gte"
         :param all:
         :return:
         """
         if (opts is None) and (all is None):
             return
-        filters = []
+        types = []
+
         if opts.bugfix or all:
-            key = {'advisory_type__' + cmp_type: 'bugfix'}
-            filters.append(query.filter(**key))
+            types.append('bugfix')
         if opts.enhancement or all:
-            key = {'advisory_type__' + cmp_type: 'enhancement'}
-            filters.append(query.filter(**key))
+            types.append('enhancement')
         if opts.newpackage or all:
-            key = {'advisory_type__' + cmp_type: 'newpackage'}
-            filters.append(query.filter(**key))
+            types.append('newpackage')
         if opts.security or all:
-            key = {'advisory_type__' + cmp_type: 'security'}
-            filters.append(query.filter(**key))
-        if opts.advisory:
-            key = {'advisory__' + cmp_type: opts.advisory}
-            filters.append(query.filter(**key))
-        if opts.bugzilla:
-            key = {'advisory_bug__' + cmp_type: opts.bugzilla}
-            filters.append(query.filter(**key))
-        if opts.cves:
-            key = {'advisory_cve__' + cmp_type: opts.cves}
-            filters.append(query.filter(**key))
-        if opts.severity:
-            key = {'advisory_severity__' + cmp_type: opts.severity}
-            filters.append(query.filter(**key))
-        self.base._update_security_filters = filters
+            types.append('security')
+
+        self.base.add_security_filters(cmp_type, types=types, advisory=opts.advisory,
+                                       bugzilla=opts.bugzilla, cves=opts.cves,
+                                       severity=opts.severity)
 
     def redirect_logger(self, stdout=None, stderr=None):
         # :api
